@@ -314,7 +314,7 @@ parse_report (const char* report_path)
 }
 
 gboolean
-upload_core (const char* uuid, const char* core_data) {
+upload_core (const char* uuid, const char* arch, const char* core_data) {
     CURL* curl = NULL;
     CURLcode result_code = 0;
     char* response_data = NULL;
@@ -322,8 +322,8 @@ upload_core (const char* uuid, const char* core_data) {
     struct curl_slist* list = NULL;
     char* crash_db_core_url = NULL;
 
-    asprintf (&crash_db_core_url, "%s/%s/submit-core/%s",
-        crash_db_url, uuid, sha512_system_uuid);
+    asprintf (&crash_db_core_url, "%s/%s/submit-core/%s/%s",
+        crash_db_url, uuid, arch, sha512_system_uuid);
 
     /* TODO use CURLOPT_READFUNCTION to transparently compress data with
      * Snappy. */
@@ -366,6 +366,7 @@ parse_and_upload_report (const char* crash_file)
     char* response_data = NULL;
     char* command = NULL;
     char* core = NULL;
+    char* arch = NULL;
     bson b[1];
 
     report = parse_report (crash_file);
@@ -386,17 +387,17 @@ parse_and_upload_report (const char* crash_file)
         if (command) {
             if (strcmp (command, "CORE") == 0) {
                 core = g_hash_table_lookup (report, "CoreDump");
-                if (core) {
-                    if (!upload_core (response_data, core)) {
-                        /* TODO handle retrying? */
+                arch = g_hash_table_lookup (report, "Architecture");
+                if (core && arch) {
+                    if (!upload_core (response_data, arch, core))
+                        /* We do not retry the upload. Once is a big enough hit
+                         * to their Internet connection, and we can always
+                         * count on the next person in line to send it. */
                         printf ("Upload of the core dump failed.\n");
-                    }
-                } else {
+                } else
                     printf ("Asked for a core dump that we don't have.\n");
-                }
-            } else {
+            } else
                 printf ("Got command: %s\n", command);
-            }
         }
     }
 
@@ -410,25 +411,44 @@ parse_and_upload_report (const char* crash_file)
 void
 create_file (const char* upload)
 {
+    /* TODO why are we strdup'ing this, rather than just leaving it as const on
+     * the g_queue? */
     char* upload_file = g_strdup (upload);
     char* crash_file = upload_to_crash_file (upload_file);
+    char* uploaded_file = NULL;
+    int fd = -1;
+    asprintf (&uploaded_file, "%sed", upload_file);
 
-    if (g_file_test (crash_file, G_FILE_TEST_EXISTS)) {
-        g_message ("%s exists", crash_file);
-        if (online_state && parse_and_upload_report (crash_file)) {
-            if (g_unlink (upload_file))
-                g_warning ("Unable to remove: %s", upload_file);
-            free (crash_file);
-        } else {
-            g_warning ("Adding to queue: %s", upload_file);
-            g_queue_push_head (report_queue, (gpointer)upload_file);
-        }
-    } else {
-        /* Already cleaned up? Nothing more we can do. */
-        if (g_unlink (upload_file))
-            g_warning ("Unable to remove: %s", upload_file);
-        free (crash_file);
+    /* We've already handled this. */
+    if (g_file_test (uploaded_file, G_FILE_TEST_EXISTS))
+        goto out;
+
+    /* Already cleaned up? Nothing more we can do. */
+    if (!g_file_test (crash_file, G_FILE_TEST_EXISTS)) {
+        if ((fd = creat (uploaded_file, 0600)) < 0)
+            g_warning ("Unable to create %s: %s", uploaded_file, strerror (errno));
+        goto out;
     }
+
+    g_message ("%s exists", crash_file);
+    if (online_state && parse_and_upload_report (crash_file)) {
+        if ((fd = creat (uploaded_file, 0600)) < 0)
+            g_warning ("Unable to create %s: %s", uploaded_file, strerror (errno));
+        goto out;
+    } else {
+        g_warning ("Adding to queue: %s", upload_file);
+        g_queue_push_head (report_queue, (gpointer)upload_file);
+        /* Don't free the upload file, as we've pushed it onto the queue */
+        free (crash_file);
+        free (uploaded_file);
+        return;
+    }
+
+    out:
+        close (fd);
+        free (crash_file);
+        free (uploaded_file);
+        free (upload_file);
 }
 
 gboolean
@@ -480,19 +500,27 @@ gboolean
 process_queue (void) {
     g_warning ("Processing queue.");
     GList* list = NULL;
-    char *upload_file, *crash_file = NULL;
+    char *upload_file, *crash_file, *uploaded_file = NULL;
+    int fd = -1;
     list = report_queue->head;
     while (list) {
         GList* next = list->next;
         upload_file = list->data;
         if (g_file_test (upload_file, G_FILE_TEST_EXISTS)) {
             crash_file = upload_to_crash_file (upload_file);
-            if (online_state && parse_and_upload_report (crash_file)) {
-                if (g_unlink (upload_file))
-                    g_warning ("Unable to remove: %s", upload_file);
+            asprintf (&uploaded_file, "%sed", upload_file);
+
+            /* We've already handled this. */
+            if (g_file_test (uploaded_file, G_FILE_TEST_EXISTS))
+                remove_from_report_queue (upload_file);
+
+            else if (online_state && parse_and_upload_report (crash_file)) {
+                if ((fd = creat (uploaded_file, 0600)) < 0)
+                    g_warning ("Unable to create %s: %s", uploaded_file, strerror (errno));
                 remove_from_report_queue (upload_file);
             }
             free (crash_file);
+            free (uploaded_file);
         } else {
             remove_from_report_queue (upload_file);
         }
@@ -506,26 +534,36 @@ process_existing_files (void)
 {
     GDir* dir = NULL;
     const gchar *file, *ext = NULL;
-    gchar* upload_file = NULL;
-    char* crash_file = NULL;
+    char *upload_file, *crash_file, *uploaded_file = NULL;
+    int fd = -1;
 
     dir = g_dir_open ("/var/crash", 0, NULL);
     while ((file = g_dir_read_name (dir)) != NULL) {
         upload_file = g_build_filename ("/var/crash", file, NULL);
-        ext = strrchr(upload_file, '.');
+        ext = strrchr (upload_file, '.');
         if (ext && strcmp(++ext, "upload") == 0) {
             crash_file = upload_to_crash_file (upload_file);
-            if (online_state && parse_and_upload_report (crash_file)) {
-                if (g_unlink (upload_file))
-                    g_warning ("Unable to remove: %s", upload_file);
-                free (crash_file);
-                free (upload_file);
+            asprintf (&uploaded_file, "%sed", upload_file);
+
+            /* We've already handled this. */
+            if (g_file_test (uploaded_file, G_FILE_TEST_EXISTS))
+                remove_from_report_queue (upload_file);
+
+            else if (online_state && parse_and_upload_report (crash_file)) {
+                if ((fd = creat (uploaded_file, 0600)) < 0)
+                    g_warning ("Unable to create %s: %s", uploaded_file, strerror (errno));
             } else {
                 g_queue_push_head (report_queue, upload_file);
+                free (uploaded_file);
+                free (crash_file);
+                /* Don't free the upload file, as we've pushed it onto the
+                 * queue */
+                continue;
             }
-        } else {
-            free (upload_file);
+            free (uploaded_file);
+            free (crash_file);
         }
+        free (upload_file);
     }
     g_dir_close (dir);
 }
@@ -648,50 +686,10 @@ get_crash_db_url (void)
         return NULL;
 }
 
-static void
-create_namespace (void)
-{
-    /* We're going to override +t globally, so let's play it safe and restrict
-     * ourselves to only being able to write in /var/crash. */
-
-    mkdir ("/var/tmp/whoopsie", 0755);
-
-    /* Set up a private mount namespace. */
-    if (unshare (CLONE_NEWNS) == -1)
-        g_error ("CLONE_NEWNS failed.");
-
-    if (mount ("/", "/var/tmp/whoopsie", NULL, MS_BIND | MS_REC | MS_RDONLY, NULL))
-        g_error ("Could not bind mount /.");
-
-    if (mount ("/var/crash", "/var/tmp/whoopsie/var/crash", NULL, MS_BIND, NULL))
-        g_error ("Could not rw mount /var/crash.");
-
-    if (chroot ("/var/tmp/whoopsie"))
-        g_error ("Could not chroot.");
-
-    if (chdir ("/"))
-        g_error ("Could not chdir to /.");
-
-    /* We don't need to worry about unmounting the above bind mounts, as once
-     * we leave the namespace, they will be released:
-     * http://lxr.linux.no/linux+v3.2.1/fs/namespace.c#L2736 */
-}
-
 static void
 drop_privileges (void)
 {
     struct passwd *pw = NULL;
-    cap_t cap;
-    /* Specify that we want to ignore the directory sticky bit */
-    cap_value_t cap_list[] = {CAP_FOWNER};
-
-    if (!CAP_IS_SUPPORTED (CAP_SETFCAP))
-        g_error ("SETFCAP is not supported.");
-
-
-    /* Ensure that we don't lose the capabilities when we drop privileges */
-    if (prctl (PR_SET_KEEPCAPS, 1) < 0)
-        g_error ("prctl failed.");
 
     if (!(pw = getpwnam (username)))
         g_error ("Failed to find user: %s", username);
@@ -705,23 +703,6 @@ drop_privileges (void)
 
     setenv ("USER", username, 1);
     setenv ("USERNAME", username, 1);
-    
-    /* Now drop all capabilities but CAP_SETFCAP and CAP_FOWNER */
-    cap = cap_init ();
-    if (cap == NULL)
-        g_error ("cap_get_proc failed.");
-    if (cap_set_flag (cap, CAP_EFFECTIVE, 1, cap_list, CAP_SET) == -1)
-        g_error ("cap_set_flag CAP_EFFECTIVE failed.");
-    if (cap_set_flag (cap, CAP_PERMITTED, 1, cap_list, CAP_SET) == -1)
-        g_error ("cap_set_flag CAP_PERMITTED failed.");
-    if (cap_set_proc (cap) == -1)
-        g_error ("cap_set_proc failed.");
-    cap_free (cap);
-
-    cap_clear (cap);
-    cap = cap_get_proc ();
-    g_warning ("capabilities: %s\n", cap_to_text(cap, NULL));
-    cap_free (cap);
 }
 
 void
@@ -741,6 +722,8 @@ setup_network_changed (void)
     GNetworkMonitor* nm = NULL;
     GSocketConnectable *addr = NULL;
     addr = g_network_address_parse_uri (crash_db_url, 80, NULL);
+    if (addr == NULL)
+        return;
 
     nm = g_network_monitor_get_default ();
     if (!nm)
@@ -779,7 +762,6 @@ main (int argc, char** argv)
     }
     free (system_uuid);
 
-    create_namespace ();
     drop_privileges ();
     exit_if_already_running ();
 
